@@ -4,6 +4,7 @@ implementing the methods and behaviors for an AMQP Channel.
 """
 import collections
 import logging
+import warnings
 
 import pika.frame as frame
 import pika.exceptions as exceptions
@@ -134,7 +135,7 @@ class Channel(object):
         """
         if not self.is_open:
             raise exceptions.ChannelClosed()
-        return self._rpc(spec.Basic.Ack(delivery_tag, multiple))
+        return self._send_method(spec.Basic.Ack(delivery_tag, multiple))
 
     def basic_cancel(self, callback=None, consumer_tag='', nowait=False):
         """This method cancels a consumer. This does not affect already
@@ -166,7 +167,8 @@ class Channel(object):
         self._rpc(spec.Basic.Cancel(consumer_tag=consumer_tag,
                                     nowait=nowait),
                   self._on_cancelok,
-                  [spec.Basic.CancelOk] if nowait is False else [])
+                  [(spec.Basic.CancelOk,
+                    {'consumer_tag': consumer_tag})] if nowait is False else [])
 
     def basic_consume(self, consumer_callback, queue='', no_ack=False,
                       exclusive=False, consumer_tag=None):
@@ -179,10 +181,12 @@ class Channel(object):
         http://www.rabbitmq.com/amqp-0-9-1-reference.html#basic.consume
 
         :param method consumer_callback: The method to callback when consuming
-        :param str|unicode queue: The queue to consume from
+        :param queue: The queue to consume from
+        :type queue: str or unicode
         :param bool no_ack: Tell the broker to not expect a response
         :param bool exclusive: Don't allow other consumers on the queue
-        :param str|unicode consumer_tag: Specify your own consumer tag
+        :param consumer_tag: Specify your own consumer tag
+        :type consumer_tag: str or unicode
         :rtype: str
 
         """
@@ -203,7 +207,8 @@ class Channel(object):
                                      no_ack=no_ack,
                                      exclusive=exclusive),
                            self._on_eventok,
-                           [spec.Basic.ConsumeOk])
+                           [(spec.Basic.ConsumeOk,
+                             {'consumer_tag': consumer_tag})])
 
         return consumer_tag
 
@@ -219,7 +224,8 @@ class Channel(object):
         http://www.rabbitmq.com/amqp-0-9-1-reference.html#basic.get
 
         :param method callback: The method to callback with a message
-        :param str|unicode queue: The queue to get a message from
+        :param queue: The queue to get a message from
+        :type queue: str or unicode
         :param bool no_ack: Tell the broker to not expect a reply
 
         """
@@ -249,7 +255,8 @@ class Channel(object):
         """
         if not self.is_open:
             raise exceptions.ChannelClosed()
-        return self._rpc(spec.Basic.Nack(delivery_tag, multiple, requeue))
+        return self._send_method(spec.Basic.Nack(delivery_tag, multiple,
+                                                 requeue))
 
     def basic_publish(self, exchange, routing_key, body,
                       properties=None, mandatory=False, immediate=False):
@@ -258,9 +265,12 @@ class Channel(object):
 
         http://www.rabbitmq.com/amqp-0-9-1-reference.html#basic.publish
 
-        :param str exchange: The exchange name
-        :param str routing_key: The routing key
-        :param str body: The message body
+        :param exchange: The exchange to publish to
+        :type exchange: str or unicode
+        :param routing_key: The routing key to bind on
+        :type routing_key: str or unicode
+        :param body: The message body
+        :type body: str or unicode
         :param pika.spec.Properties properties: Basic.properties
         :param bool mandatory: The mandatory flag
         :param bool immediate: The immediate flag
@@ -327,7 +337,7 @@ class Channel(object):
         """
         if not self.is_open:
             raise exceptions.ChannelClosed()
-        return self._rpc(spec.Basic.Reject(delivery_tag, requeue))
+        return self._send_method(spec.Basic.Reject(delivery_tag, requeue))
 
     def basic_recover(self, callback=None, requeue=False):
         """This method asks the server to redeliver all unacknowledged messages
@@ -409,9 +419,12 @@ class Channel(object):
         """Bind an exchange to another exchange.
 
         :param method callback: The method to call on Exchange.BindOk
-        :param str|unicode destination: The destination exchange to bind
-        :param str|unicode source: The source exchange to bind to
-        :param str|unicode routing_key: The routing key to bind on
+        :param destination: The destination exchange to bind
+        :type destination: str or unicode
+        :param source: The source exchange to bind to
+        :type source: str or unicode
+        :param routing_key: The routing key to bind on
+        :type routing_key: str or unicode
         :param bool nowait: Do not wait for an Exchange.BindOk
         :param dict arguments: Custom key/value pair arguments for the binding
 
@@ -425,10 +438,10 @@ class Channel(object):
     def exchange_declare(self, callback=None, exchange=None,
                          exchange_type='direct', passive=False, durable=False,
                          auto_delete=False, internal=False, nowait=False,
-                         arguments=None):
+                         arguments=None, type=None):
         """This method creates an exchange if it does not already exist, and if
         the exchange exists, verifies that it is of the correct and expected
-         class.
+        class.
 
         If passive set, the server will reply with Declare-Ok if the exchange
         already exists with the same name, and raise an error if not and if the
@@ -436,7 +449,8 @@ class Channel(object):
         exception with reply code 404 (not found).
 
         :param method callback: Call this method on Exchange.DeclareOk
-        :param str|unicode exchange: The exchange name consists of a non-empty
+        :param exchange: The exchange name consists of a non-empty
+        :type exchange: str or unicode
                                      sequence of these characters: letters,
                                      digits, hyphen, underscore, period, or
                                      colon.
@@ -447,9 +461,15 @@ class Channel(object):
         :param bool internal: Can only be published to by other exchanges
         :param bool nowait: Do not expect an Exchange.DeclareOk response
         :param dict arguments: Custom key/value pair arguments for the exchange
+        :param str type: The deprecated exchange type parameter
 
         """
         self._validate_channel_and_callback(callback)
+        if type is not None:
+            warnings.warn('type is deprecated, use exchange_type instead',
+                          DeprecationWarning)
+            if exchange_type == 'direct' and type != exchange_type:
+                exchange_type = type
         return self._rpc(spec.Exchange.Declare(0, exchange, exchange_type,
                                                passive, durable, auto_delete,
                                                internal, nowait,
@@ -462,7 +482,8 @@ class Channel(object):
         """Delete the exchange.
 
         :param method callback: The method to call on Exchange.DeleteOk
-        :param str|unicode exchange: The exchange name
+        :param exchange: The exchange name
+        :type exchange: str or unicode
         :param bool if_unused: only delete if the exchange is unused
         :param bool nowait: Do not wait for an Exchange.DeleteOk
 
@@ -477,9 +498,12 @@ class Channel(object):
         """Unbind an exchange from another exchange.
 
         :param method callback: The method to call on Exchange.UnbindOk
-        :param str|unicode destination: The destination exchange to unbind
-        :param str|unicode source: The source exchange to unbind from
-        :param str|unicode routing_key: The routing key to unbind
+        :param destination: The destination exchange to unbind
+        :type destination: str or unicode
+        :param source: The source exchange to unbind from
+        :type source: str or unicode
+        :param routing_key: The routing key to unbind
+        :type routing_key: str or unicode
         :param bool nowait: Do not wait for an Exchange.UnbindOk
         :param dict arguments: Custom key/value pair arguments for the binding
 
@@ -546,9 +570,12 @@ class Channel(object):
         """Bind the queue to the specified exchange
 
         :param method callback: The method to call on Queue.BindOk
-        :param str|unicode queue: The queue to bind to the exchange
-        :param str|unicode exchange: The source exchange to bind to
-        :param str|unicode routing_key: The routing key to bind on
+        :param queue: The queue to bind to the exchange
+        :type queue: str or unicode
+        :param exchange: The source exchange to bind to
+        :type exchange: str or unicode
+        :param routing_key: The routing key to bind on
+        :type routing_key: str or unicode
         :param bool nowait: Do not wait for a Queue.BindOk
         :param dict arguments: Custom key/value pair arguments for the binding
 
@@ -559,7 +586,7 @@ class Channel(object):
                                          nowait, arguments or dict()), callback,
                          replies)
 
-    def queue_declare(self, callback, queue, passive=False, durable=False,
+    def queue_declare(self, callback, queue='', passive=False, durable=False,
                       exclusive=False, auto_delete=False, nowait=False,
                       arguments=None):
         """Declare queue, create if needed. This method creates or checks a
@@ -567,8 +594,11 @@ class Channel(object):
         properties that control the durability of the queue and its contents,
         and the level of sharing for the queue.
 
+        Leave the queue name empty for a auto-named queue in RabbitMQ
+
         :param method callback: The method to call on Queue.DeclareOk
-        :param str|unicode queue: The queue name
+        :param queue: The queue name
+        :type queue: str or unicode
         :param bool passive: Only check to see if the queue exists
         :param bool durable: Survive reboots of the broker
         :param bool exclusive: Only allow access by the current connection
@@ -577,7 +607,8 @@ class Channel(object):
         :param dict arguments: Custom key/value arguments for the queue
 
         """
-        replies = [spec.Queue.DeclareOk] if nowait is False else []
+        replies = [(spec.Queue.DeclareOk,
+                    {'queue': queue})] if nowait is False else []
         self._validate_channel_and_callback(callback)
         return self._rpc(spec.Queue.Declare(0, queue, passive, durable,
                                             exclusive, auto_delete, nowait,
@@ -589,7 +620,8 @@ class Channel(object):
         """Delete a queue from the broker.
 
         :param method callback: The method to call on Queue.DeleteOk
-        :param str|unicode queue: The queue to delete
+        :param queue: The queue to delete
+        :type queue: str or unicode
         :param bool if_unused: only delete if it's unused
         :param bool if_empty: only delete if the queue is empty
         :param bool nowait: Do not wait for a Queue.DeleteOk
@@ -605,7 +637,8 @@ class Channel(object):
         """Purge all of the messages from the specified queue
 
         :param method callback: The method to call on Queue.PurgeOk
-        :param str|unicode: The queue to purge
+        :param queue: The queue to purge
+        :type queue: str or unicode
         :param bool nowait: Do not expect a Queue.PurgeOk response
 
         """
@@ -619,9 +652,12 @@ class Channel(object):
         """Unbind a queue from an exchange.
 
         :param method callback: The method to call on Queue.UnbindOk
-        :param str|unicode queue: The queue to unbind from the exchange
-        :param str|unicode exchange: The source exchange to bind from
-        :param str|unicode routing_key: The routing key to unbind
+        :param queue: The queue to unbind from the exchange
+        :type queue: str or unicode
+        :param exchange: The source exchange to bind from
+        :type exchange: str or unicode
+        :param routing_key: The routing key to unbind
+        :type routing_key: str or unicode
         :param dict arguments: Custom key/value pair arguments for the binding
 
         """
@@ -696,7 +732,8 @@ class Channel(object):
         :param str consumer_tag: The consumer tag for the message
         :param pika.frame.Method method_frame: The received method frame
         :param pika.frame.Header header_frame: The received header frame
-        :param str|unicode body: The message body
+        :param body: The message body
+        :type body: str or unicode
 
         """
         self._pending[consumer_tag].append((self, method_frame.method,
@@ -726,7 +763,11 @@ class Channel(object):
         :param pika.amqp_object.Frame frame_value: The frame to deliver
 
         """
-        response = self.frame_dispatcher.process(frame_value)
+        try:
+            response = self.frame_dispatcher.process(frame_value)
+        except exceptions.UnexpectedFrameError:
+            return self._unexpected_frame(frame_value)
+
         if response:
             if isinstance(response[0].method, spec.Basic.Deliver):
                 self._on_deliver(*response)
@@ -787,7 +828,8 @@ class Channel(object):
 
         :param pika.frame.Method method_frame: The method frame received
         :param pika.frame.Header header_frame: The header frame received
-        :param str body: The body received
+        :param body: The body received
+        :type body: str or unicode
 
         """
         LOGGER.debug('Called with %r, %r, %r', method_frame, header_frame, body)
@@ -850,7 +892,8 @@ class Channel(object):
 
         :param pika.frame.Method method_frame: The method frame received
         :param pika.frame.Header header_frame: The header frame received
-        :param str body: The body received
+        :param body: The body received
+        :type body: str or unicode
 
         """
         if self._on_getok_callback is not None:
@@ -881,7 +924,8 @@ class Channel(object):
 
         :param pika.frame.Method method_frame: The Basic.Return frame
         :param pika.frame.Header header_frame: The content header frame
-        :param str|unicode body: The message body
+        :param body: The message body
+        :type body: str or unicode
 
         """
         if not self.callbacks.process(self.channel_number, '_on_return',
@@ -946,10 +990,17 @@ class Channel(object):
         # If acceptable replies are set, add callbacks
         if acceptable_replies:
             for reply in acceptable_replies or list():
+                if isinstance(reply, tuple):
+                    reply, arguments = reply
+                else:
+                    arguments = None
+                LOGGER.debug('Adding in on_synchronous_complete callback')
                 self.callbacks.add(self.channel_number, reply,
-                                   self._on_synchronous_complete)
+                                   self._on_synchronous_complete,
+                                   arguments=arguments)
                 if callback:
-                    self.callbacks.add(self.channel_number, reply, callback)
+                    LOGGER.debug('Adding passed in callback')
+                    self.callbacks.add(self.channel_number, reply, callback, arguments=arguments)
 
         self._send_method(method_frame)
 
@@ -981,6 +1032,9 @@ class Channel(object):
         self._send_method(spec.Channel.Close(self._reply_code,
                                              self._reply_text, 0, 0))
 
+    def _unexpected_frame(self, frame_value):
+        LOGGER.warning('Unexpected frame: %r', frame_value)
+
     def _validate_channel_and_callback(self, callback):
         if not self.is_open:
             raise exceptions.ChannelClosed()
