@@ -557,6 +557,7 @@ class Connection(object):
         :param method on_open_callback: The callback when the channel is opened
         :param int channel_number: The channel number to use, defaults to the
                                    next available.
+        :rtype: pika.channel.Channel
 
         """
         if not channel_number:
@@ -565,6 +566,7 @@ class Connection(object):
                                                               on_open_callback)
         self._add_channel_callbacks(channel_number)
         self._channels[channel_number].open()
+        return self._channels[channel_number]
 
     def close(self, reply_code=200, reply_text='Normal shutdown'):
         """Disconnect from RabbitMQ. If there are any open channels, it will
@@ -1050,8 +1052,9 @@ class Connection(object):
         if not from_adapter:
             self._adapter_disconnect()
         self._set_connection_state(self.CONNECTION_CLOSED)
-        for channel in self._channels:
-            self._channels[channel].on_remote_close(method_frame)
+        if method_frame:
+            for channel in self._channels:
+                self._channels[channel].on_remote_close(method_frame)
         self._process_connection_closed_callbacks()
         self._remove_connection_callbacks()
 
@@ -1173,6 +1176,7 @@ class Connection(object):
         self.frames_received += 1
 
         # Process any callbacks, if True, exit method
+        LOGGER.debug('Frame: %r', frame_value)
         if self._process_callbacks(frame_value):
             return
 
@@ -1306,12 +1310,11 @@ class Connection(object):
         """
         if self.is_closed:
             raise exceptions.ConnectionClosed
+        LOGGER.debug('Frame: %r', frame_value)
         marshaled_frame = frame_value.marshal()
         self.bytes_sent += len(marshaled_frame)
         self.frames_sent += 1
         self.outbound_buffer.write(marshaled_frame)
-        LOGGER.debug('Added %i bytes to the outbound buffer',
-                     len(marshaled_frame))
         self._flush_outbound()
         if self.params.backpressure_detection:
             self._detect_backpressure()
